package org.steveshipway.cheshirecat;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.steveshipway.cheshirecat.directive.Directive;
import org.steveshipway.cheshirecat.ChunkCoordinates;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.util.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;

public class Blueprint {
	private boolean floaty = false;  // float y coordinate before build
	private boolean recurse = false; // allow recursion
	private int distance = 2;        // not used
	public boolean visible = true;   // hidden or visible blueprint
	private LinkedList steps = null; // build steps
	public String name;              // my name
	public String helptext = "No description available";
	
	public Blueprint(String name, String description, Boolean v, LinkedList steps) {
		// create this blueprint
		this.name = name;
		this.steps = steps;
		this.visible = v;
		if( description != null && description != "" ) {
			this.helptext = description;
		}
		if( steps.isEmpty() ) {
        	CheshireCat.logger.warn("["+name+"] Created a blueprint with no steps?");
        	return;
		}
		// iterate over the commands, removing any options and setting the flags
		Iterator i = this.steps.iterator();
	    while(i.hasNext()) {
	        Directive d = (Directive)i.next();
	        String s = d.description();
	        if( s != null ) {
	        	this.helptext = s;
	        	// this.steps.remove(d);
	        	continue;
	        }
	        s = d.option();
	        if( s != null ) {
	        	if( s.contains("float")) {
	        		this.floaty = true;
	        	}
	        	if( s.contains("recurs")) { // catch recurse and recursive
	        		this.recurse = true;
	        	}

	        	// this.steps.remove(d);
	        	continue;
	        }
	        int n = d.distance();
	        if( n >= 0 ) {
	        	this.distance = n;
	        	continue;
	        }
	    }
    	//CheshireCat.logger.info("Created a blueprint with "+this.steps.size()+" steps");
	}
	
	public boolean isVisible() {
		return visible;
	}
	public boolean setVisible(boolean v) {
		visible = v;
		return visible;
	}

	public String helpText() {
		if( this.helptext != null ) {
			return this.helptext;
		}
		return "No description available";
	}
	public String name() { return this.name; }
	
	// build this blueprint, at the specified coordinates.
	public boolean build(World world, ChunkCoordinates location, int r ) {
		// iterate over steps, actioning each in turn
		// may require (gulp) recursion...
		// return true if it was successfully built
		boolean rv = true;
		if( world.isRemote ) {
        	CheshireCat.logger.warn("Blueprint build called with remote world!  Should never happen.");
			return false;			
		}
		if( this.steps == null || this.steps.size() == 0 ) {
        	CheshireCat.logger.warn("Build of '"+this.name+"' had no steps!  This seems wrong...");
			return false;
		}
    	CheshireCat.logger.info("Build of blueprint '"+this.name+"' starting: "+this.steps.size()+" steps");
		
    	// floating option -- go up.down to find surface of world
    	if( this.floaty ) {
    		BlockPos blk = new BlockPos(location.posX, location.posY-1, location.posZ);
        	Block b = world.getChunkFromBlockCoords(blk).getBlock(blk);
    		while( location.posY > 0 && b == Blocks.air ) {
    			location.posY -= 1;
    			blk = blk.down();
    			b = world.getChunkFromBlockCoords(blk).getBlock(blk);
    		}
    		while( location.posY < 128 && b != Blocks.air && b != Blocks.water ) {
    			location.posY += 1; 
       			blk = blk.up();
    			b = world.getChunkFromBlockCoords(blk).getBlock(blk);
    		}
        	CheshireCat.logger.info("Floating Y coordinate to surface level: "+location.posY);
    	}
    	
		Iterator i = this.steps.iterator();
	    while(i.hasNext()) {
	        Directive d = (Directive)i.next();
	    	//CheshireCat.logger.debug("Calling step: "+d.asText());
	        rv = d.build(world, location, r);
	        if(!rv) {
	        	CheshireCat.logger.error("Build of '"+this.name+"' failed with step: ["+d.asText()+"]");
	        	break; 
	        }
	    }
    	//CheshireCat.logger.info("Build of blueprint '"+this.name+"' ended");
		return rv;
	}
	public String asText() {
		String s = "";
		Iterator i = this.steps.iterator();
		if( visible ) {
			s = "PUBLIC";
		} else {
			s = "PRIVATE";
		}
		s = s + " PLAN " + name + " " + helptext + "\n";
		if( this.floaty ) {
			s = s + " OPTION float\n";
		}
		if( this.recurse ) {
			s = s + " OPTION recursive\n";
		}
	    while(i.hasNext()) {
	        Directive d = (Directive)i.next();
	        s = s + "  " + d.asText() + "\n";
	    }
	    s = s + "END\n";
		return s;
	}
}
